   /////////////////////////////////////////////////////////////////
   //-------------------------------------------------------------//
   //-------------------------------------------------------------//
   //-------------------------------------------------------------//                
   //-------Windgeschwinidigkeistmessung mittels ein Hallsensor---//				
   //-------------------------------------------------------------//               
   //-------------------------------------------------------------//			  
   //-------max.Geschwindigkeit 50 m/s = 238,3 Hz-----------------//			   
   //-------------------------------------------------------------//
   //---------------------------------()--------------------------//
   //------------------------------()-/---------------------------//	 
   //---------Schalensternanemometer--|---------------------------//
   //---------------------------------()--------------------------//
   //-------------------------------------------------------------//						
   /////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------
//-------------Bibliotheken einbinden--------------------------------------
//-------------------------------------------------------------------------
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <util/twi.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include <avr/wdt.h>
#include "lcd-routines.h"
//#include TWI_Slave.h


//------------------------------------------------------------------------
//------------Prozessortakt einstellen------------------------------------
//------------------------------------------------------------------------
#ifndef F_CPU             	   
#define F_CPU 4000000 UL          
#endif


#define TWIM_LESEN       1
#define TWIM_SCHREIBEN   0

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif


#ifndef Ct					// c --> max.geschwindigkeit = 50m/s
#define Ct 020				// t --> minimale Impulszeitinterval 1/238,3 Hz = 4,1 ms 
#endif  					// C * t = 0,20 m


//------------------------------------------------------------------------------------------------------------//
//-----------------Variablen deklartion-----------------------------------------------------------------------// 
//------------------------------------------------------------------------------------------------------------//
volatile unsigned char Si_wert= 0;
volatile unsigned char Overflow   = 0; 		// Anzahl der Timer Overflows die waehrend der Messperiode passieren
volatile unsigned char Overflow_2 = 0; 		
volatile unsigned short Starttime =0;
volatile unsigned short Endtime = 0;
volatile unsigned char Durchlauf;   		// Job Flag

volatile int sekunden = 0;
volatile int minuten = 0;

unsigned short ti_neu = 0;
unsigned short ti2_neu = 0;
unsigned short ti_alt = 0;  
unsigned short Zeit_ti ;



unsigned char Geschwindigkeit_in_10min;
unsigned char Standardabweichung;

char lcd_Geschwindigkeit_String[10];
char lcd_Impulse_String[10];
char lcd_sekunden_String[10];
char lcd_Overflow_String[10];
char my_string[7];



//-------------------------------------------------------------------------------------------------------------//
//--------Interrupt fr die zeitbestimmung der einzelnen Impulse-----------------------------------------------//
//-------------------------------------------------------------------------------------------------------------//

ISR( TIMER1_CAPT_vect )
{
  
	Si_wert++;
    static unsigned char ErsteFlanke = TRUE;

  
  // Bei der ersten Flanke beginnt die Messung, es wird der momentane
  // Timer beim Input Capture als Startwert gesichert
  
  if( ErsteFlanke == TRUE )
	{
		Starttime = ICR1;		// Startzeit= ICR 1
		Overflow = 0;
		Durchlauf = TRUE;
		ErsteFlanke = FALSE;       	// Die naechste Flanke ist das Ende der Messung
	}


 // das ist die zweite Flanke im Messzyklus. Die Messung wird gestoppt
  
  else
	{
		Endtime = ICR1;		        // Stopzeit fr die Messung= ICR1
		Overflow = 0;
		Durchlauf = TRUE;  			// Eine vollst?ndige Messung. Sie kann ausgewertet werden 
		ErsteFlanke = TRUE;        	// Bei der naechsten Flanke beginnt der naechste Messzyklus
	}
}



//------------------------------------------------------------------------------------------------------//
//------------------Interrupt fr Zhler berlauf-------------------------------------------------------//
//------------------------------------------------------------------------------------------------------//	
ISR( TIMER1_OVF_vect )
{
  Overflow++;

}


//------------------------------------------------------------------------------------------------------//
//------------------Interrupt fr 10 min. erreicht------------------------------------------------------//
//------------------------------------------------------------------------------------------------------//	

ISR(TIMER1_COMPA_vect)
{
  sekunden++;
						
}

		


//---------------------------------------------------------------------------------------------		  
//-------Berechnung der zeit zwischen den Impulsen des Anemometers-----------------------------
//---------------------------------------------------------------------------------------------
void Impulse_berechnen(void)
{

	// Die Zeitdauer zwischen den Flanken bestimmen
	// Da EndTime und StartTime unsigned sind, braucht nicht
	// darauf Ruecksicht genommen werden, dass EndTime auch
	// kleiner als StartTime sein kann. Es kommt trotzdem
	// das richtige Ergebnis raus.	  
	// Die Zeitdauer wird als Anzahl der Taktzyklen zwischen den
	// beiden gemessenen Flanken berechnet 65536 --> 16bit 
	
	Zeit_ti = (Overflow*65536) + Endtime - Starttime;

	// ... mit der bekannten Taktfrequenz ergibt sich dann die Signalfrequenz
	//Zeit_ti =  ((Zeit_ti) / 62500);       	// f = 1 / t   
    

	ti_alt = Zeit_ti;

	ti_neu = ti_neu + ti_alt;
	
	ti2_neu = (ti_neu * ti_neu) + ti2_neu;
	
}
		
    	
		

//-------------------------------------------------------------------------------------------------------//
//----------------------Interrupts klar machen-----------------------------------------------------------//	
//-------------------------------------------------------------------------------------------------------//
void init_interrupt(void)			
{
	TCCR1B =  (1<<ICES1)|(1<<CS10) |(1<<CS11);		// Input Capture steigende Flanke an ICP-Pin(PD6), keine Prescale 
	TIMSK  = (1<<TICIE1)|(1<<TOIE1)|(1<<OCIE1A) ;	// Interrupts akivieren, Capture, Overflow 
	OCR1A  = 62499;
	sei();											// Interrups freigeben
}


//-------------------------------------------------------------------------------------------------------//
//----------------------Messwerte auf einem LCD ausgeben-------------------------------------------------//	
//-------------------------------------------------------------------------------------------------------//

void Messwerte_ausgeben(void)
{
 
		  
	//Windgeschwindigkeit auf dem Display ausgeben
	itoa( sekunden, lcd_sekunden_String, 10 );	// Das Ergebnis fuer die Anzeige aufbereiten 
		  
	itoa( Si_wert, lcd_Impulse_String, 10 );  
//	itoa( Overflow, lcd_Overflow_String, 10 ); 
	itoa(Starttime,lcd_Geschwindigkeit_String, 10);
	// Messergebnisse auf dem Displayausgeben
	lcd_clear();  
		
	lcd_setcursor( 0,1 );							// Gehe zu Pos. XY
	lcd_string(lcd_sekunden_String);			// Datenwert
	lcd_string("s");

	lcd_setcursor( 3,1 );				       			// Ausgabe Maske
	lcd_string(lcd_Impulse_String);			// Datenwert
	lcd_string("i");				       			// Ausgabe Maske

//	lcd_setcursor( 6,1 );				       			// Ausgabe Maske
//	lcd_string(lcd_Overflow_String);			// Datenwert
//	lcd_string("d");	

	lcd_setcursor( 0,2 );				       			// Ausgabe Maske
	
	lcd_string(lcd_Geschwindigkeit_String);			// Datenwert
	
	_delay_ms(1000);
//	lcd_string("m/s");
							// Das wars: Display ist wieder up to date
							// die naechste Messung kann starten
	
}

/*
 
Funktion zur Umwandlung einer vorzeichenbehafteten 32 Bit Zahl in einen String
 
*/
 
void my_itoa(int32_t zahl ,char* string) {       
  
  uint8_t i;
 
  string[11]='\0';                  // String Terminator
  if( zahl < 0 ) {                  // ist die Zahl negativ?
    string[0] = '-';              
    zahl = -zahl;
  }
  else string[0] = ' ';             // Zahl ist positiv
 
  for(i=10; i>=1; i--) {
    string[i]=(zahl % 10) +'0';     // Modulo rechnen, dann den ASCII-Code von '0' addieren
    zahl /= 10;
  }
}



/*
 
Funktion zur Anzeige einer 32 Bit Zahl im Stringformat
auf einem LCD mit HD44780 Controller
 
Parameter:
 
char* string  : Zeiger auf String, welcher mit my_itoa() erzeugt wurde
uint8_t start : Offset im String, ab der die Zahl ausgegeben werden soll,
                das ist notwenig wenn Zahlen mit begrenztem Zahlenbereich
                ausgegeben werden sollen
                Vorzeichenlose Zahlen      : 0..10
                Vorzeichenbehaftete zahlen : 1..11
uint8_t komma : Offset im String, zeigt auf die Stelle an welcher das virtuelle
                Komma steht (erste Nachkommastelle)
                komma muss immer grsser oder gleich start sein !
 
uint8_t frac  : Anzahl der Nachkommastellen
 
*/
 
void my_print_LCD(char* string, uint8_t start, uint8_t komma, uint8_t frac) {
 
  uint8_t i;            // Zhler
  uint8_t flag=0;       // Merker fr fhrende Nullen

  lcd_clear();
 
  // Vorzeichen ausgeben  
  if (string[0]=='-') lcd_data('-'); else lcd_data(' ');
 
  // Vorkommastellen ohne fhrende Nullen ausgeben
  for(i=start; i<komma; i++) {
    if (flag==1 || string[i]!='0') {
      lcd_data(string[i]);
      flag = 1;
    }
    else lcd_data(' ');         // Leerzeichen
  }
 
  lcd_data(',');                // Komma ausgeben
 
  // Nachkommastellen ausgeben
  for(; i<(komma+frac); i++) lcd_data(string[i]);
 
}



int main (void)
{

	lcd_init();			     		//lcd initialisieren
	init_interrupt();				//Interrups organisieren
	
	
			
	while(1)					//diese Schleife soll das Programm immer wiederholen.
	{
			
		
			
			Messwerte_ausgeben();	
		//char y_string[12]="-0134567891\0";
		//my_itoa(Zeit_ti,y_string);
		
		//my_print_LCD(y_string, 2, 5, 2);

		if( Durchlauf == TRUE)  	// liegt ein vollstndiger Durchlauf vor?
		{
			Impulse_berechnen();
		
	
			Durchlauf = FALSE;

		}
			
		
		if (sekunden == 600)
	
		{
	//	ti_rest = (600 * 65536) - ti_neu; 
	//	ti_neu  = ti_neu + ti_rest;
	//	ti2_neu = (ti_rest * ti_rest) + ti2_neu;

		Geschwindigkeit_in_10min = (Ct / Si_wert) * (ti_neu);

		Standardabweichung = (Ct * Ct)*( ((1 / ti2_neu) - (1 / Si_wert)) * ( ti_neu * ti_neu));
	
			
			
		Si_wert = 0;				// Si_wert  null setzen

		ti2_neu = 0;				// ti_neu  null setzen

		ti_neu  = 0;				// ti_neu	null setzen
			
		sekunden = 0;			    // 10 minuten zurcksetzen
			
//----------------------------------------------------------------------------------------------------------//
//--------IC-BUS aktiv warten auf Master-------------------------------------------------------------------//
//----------------------------------------------------------------------------------------------------------//
			
		//bertragungsrate auf 23040Hz setzen (s.o.)
		TWBR = 48;
		TWSR = 1; 


		//Eigene Slave-Ansprechadresse einstellen
        TWAR = (1 << 1);

		//TWI aktivieren:
        TWCR = (1<<TWEN)|(1<<TWEA);

		if(TWCR & (1<<TWINT))
			{
		//ohne Ack:
	    TWDR = Geschwindigkeit_in_10min;
		TWDR= Standardabweichung;
  	    TWCR = (1<<TWINT)|(1<<TWEN);
	        
	    while (!(TWCR & (1<<TWINT)));
		//Stop:
		TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|(1<<TWEA);
			}

		}
	}
}			
 

















  
 
      


			
























